==> dart_icalendar-1.0.1.tar.gz: lib/models/attachment_model.dart <==
import 'dart:convert';

class Attachment {
  final Uri uri;
  final String? mimeType;
  final String? fileName;

  Attachment({
    required this.uri,
    this.mimeType,
    this.fileName,
  });

  String toICalString() {
    final buffer = StringBuffer();
    buffer.write('ATTACH;');
    if (fileName != null) {
      buffer.write('FILENAME=$fileName;');
    }
    if (mimeType != null) {
      buffer.write('FMTTYPE=$mimeType');
      if (fileName != null) {
        buffer.write(':');
      }
    } else {
      buffer.write('\r\n');
    }

    if (fileName != null || mimeType == null) {
      buffer.write(uri.toString().trim());
    }
    return buffer.toString();
  }

  static String _removeNonPrintable(String str) {
    return str.replaceAll(RegExp(r'[^\x20-\x7E]'), '');
  }

  factory Attachment.fromICalString(String icalString) {
    icalString = _removeNonPrintable(icalString);
    final uriPattern = RegExp(r'https?://\S+');
    final uriMatch = uriPattern.firstMatch(icalString);
    final uriString = uriMatch != null ? uriMatch.group(0) : '';

    final mimeType = icalString.contains(';FMTTYPE=')
        ? icalString.split(';FMTTYPE=').last.split(';').first.split(':').first
        : null;

    final fileName = icalString.contains(';FILENAME=')
        ? Uri.decodeComponent(icalString
            .split(';FILENAME=')
            .last
            .split(';')
            .first
            .split(':')
            .first)
        : null;

    if (uriString == '') {
      throw ArgumentError('The iCal string does not contain a valid URI.');
    }

    return Attachment(
      uri: Uri.parse(uriString ?? ''),
      mimeType: mimeType,
      fileName: fileName,
    );
  }

  Attachment copyWith({
    Uri? uri,
    String? mimeType,
    String? fileName,
  }) {
    return Attachment(
      uri: uri ?? this.uri,
      mimeType: mimeType ?? this.mimeType,
      fileName: fileName ?? this.fileName,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'uri': uri.toString(),
      'mimeType': mimeType,
      'fileName': fileName,
    };
  }

  factory Attachment.fromMap(Map<String, dynamic> map) {
    return Attachment(
      uri: Uri.parse(map['uri']),
      mimeType: map['mimeType'],
      fileName: map['fileName'],
    );
  }

  String toJson() => json.encode(toMap());

  factory Attachment.fromJson(String source) =>
      Attachment.fromMap(json.decode(source));

  @override
  String toString() =>
      'Attachment(uri: $uri, mimeType: $mimeType, fileName: $fileName)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is Attachment &&
        other.uri == uri &&
        other.mimeType == mimeType &&
        other.fileName == fileName;
  }

  @override
  int get hashCode => uri.hashCode ^ mimeType.hashCode ^ fileName.hashCode;
}

==> dart_icalendar-1.0.2.tar.gz: lib/models/attachment_model.dart <==
import 'dart:convert';

class Attachment {
  final Uri uri;
  final String? mimeType;
  final String? fileName;

  Attachment({
    required this.uri,
    this.mimeType,
    this.fileName,
  });

  String toICalString() {
    final buffer = StringBuffer();
    buffer.write('ATTACH;');
    if (fileName != null) {
      buffer.write('FILENAME=$fileName;');
    }
    if (mimeType != null) {
      buffer.write('FMTTYPE=$mimeType');
      if (fileName != null) {
        buffer.write(':');
      }
    } else {
      buffer.write('\r\n');
    }

    if (fileName != null || mimeType == null) {
      buffer.write(uri.toString().trim());
    }
    return buffer.toString();
  }

  static String _removeNonPrintable(String str) {
    return str.replaceAll(RegExp(r'[^\x20-\x7E]'), '');
  }

  factory Attachment.fromICalString(String icalString) {
    icalString = _removeNonPrintable(icalString);
    final uriPattern = RegExp(r'https?://\S+');
    final uriMatch = uriPattern.firstMatch(icalString);
    final uriString = uriMatch != null ? uriMatch.group(0) : '';

    final mimeType = icalString.contains(';FMTTYPE=')
        ? icalString.split(';FMTTYPE=').last.split(';').first.split(':').first
        : null;

    final fileName = icalString.contains(';FILENAME=')
        ? Uri.decodeComponent(icalString
            .split(';FILENAME=')
            .last
            .split(';')
            .first
            .split(':')
            .first)
        : null;

    if (uriString == '') {
      throw ArgumentError('The iCal string does not contain a valid URI.');
    }

    return Attachment(
      uri: Uri.parse(uriString ?? ''),
      mimeType: mimeType,
      fileName: fileName,
    );
  }

  Attachment copyWith({
    Uri? uri,
    String? mimeType,
    String? fileName,
  }) {
    return Attachment(
      uri: uri ?? this.uri,
      mimeType: mimeType ?? this.mimeType,
      fileName: fileName ?? this.fileName,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'uri': uri.toString(),
      'mime_type': mimeType,
      'file_name': fileName,
    };
  }

  factory Attachment.fromMap(Map<String, dynamic> map) {
    return Attachment(
      uri: Uri.parse(map['uri']),
      mimeType: map['mime_type'],
      fileName: map['file_name'],
    );
  }

  String toJson() => json.encode(toMap());

  factory Attachment.fromJson(String source) =>
      Attachment.fromMap(json.decode(source));

  @override
  String toString() =>
      'Attachment(uri: $uri, mimeType: $mimeType, fileName: $fileName)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is Attachment &&
        other.uri == uri &&
        other.mimeType == mimeType &&
        other.fileName == fileName;
  }

  @override
  int get hashCode => uri.hashCode ^ mimeType.hashCode ^ fileName.hashCode;
}
